@@ -349,14 +349,14 @@ void Config::load(const std::string& file)
 	char *text;
 	gsize length;
 
-	g_file_get_contents (file.c_str(), &text, &length, &err);
-	gerr_to_consistency(err, "loading file " + file);
+    g_file_get_contents (file.c_str(), &text, &length, &err);
+    gerr_to_consistency(err, "loading file " + file);
 
 	try {
 		if (length > 0 && text[0] == '<')
 			// Old format
 			loadOld(text, length, file, *this);
-		else
+		else if (length > 0)
 		{
 			g_key_file_load_from_file(cfg, file.c_str(), G_KEY_FILE_NONE, &err);
 			gerr_to_consistency(err, "parsing .ini file " + file);
@@ -382,13 +382,16 @@ void Config::save(const std::string& file)
 	if (!data)
 		throw wibble::exception::Consistency("serialising configuration", "g_key_file_to_data failed");
 
-	try {
-		g_file_set_contents(file.c_str(), data, length, &err);
-		gerr_to_consistency(err, "saving file " + file);
-	} catch (...) {
-		g_free(data);
-		throw;
-	}
+    if (length > 0)
+    {
+        try {
+            g_file_set_contents(file.c_str(), data, length, &err);
+            gerr_to_consistency(err, "saving file " + file);
+        } catch (...) {
+            g_free(data);
+            throw;
+        }
+    }
 
 	g_free(data);
 }
