@@ -1,7 +1,7 @@
 /*
  * Mailbox folder access
  *
- * Copyright (C) 2004--2008  Enrico Zini <enrico@debian.org>
+ * Copyright (C) 2004--2011  Enrico Zini <enrico@debian.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,6 +20,7 @@
 
 #include <buffy/mailfolder/mailbox.h>
 #include <wibble/exception.h>
+#include <wibble/sys/fs.h>
 #include <buffy/utils/system.h>
 #include <buffy/utils/from.h>
 
@@ -105,13 +106,13 @@ static int parse_mime_header (gzFile in, char* buf, int bufsize)
 
 bool Mailbox::changed()
 {
-	struct stat st;
-	if (!statIfFound(_path, &st))
-		if (! _deleted)
-		{
-			_deleted = true;
-			return true;
-		}
+    std::auto_ptr<struct stat> st = sys::fs::stat(_path);
+    if (!st.get())
+        if (! _deleted)
+        {
+            _deleted = true;
+            return true;
+        }
 
 	if (_deleted)
 	{
@@ -119,7 +120,7 @@ bool Mailbox::changed()
 		return true;
 	}
 
-	return st.st_mtime > _mbox_mtime || st.st_size != _mbox_size;
+    return st->st_mtime > _mbox_mtime || st->st_size != _mbox_size;
 }
 
 void Mailbox::updateStatistics()
@@ -131,36 +132,36 @@ void Mailbox::updateStatistics()
 	gzFile in = 0;
 
 	/// Count messages in the 'new' directory
-	
-	// Perform consistency checks on the directory
-	struct stat st;
-	if (!statIfFound(_path, &st))
-	{
-		_stat_total = 0;
-		_stat_unread = 0;
-		_stat_new = 0;
-		_stat_flagged = 0;
-		_deleted = true;
-		return;
-	}
-	if (S_ISDIR(st.st_mode) != 0)
-		throw wibble::exception::Consistency(_path + " is a directory");
 
-	if (_deleted)
-		_deleted = false;
+    // Perform consistency checks on the directory
+    std::auto_ptr<struct stat> st = sys::fs::stat(_path);
+    if (!st.get())
+    {
+        _stat_total = 0;
+        _stat_unread = 0;
+        _stat_new = 0;
+        _stat_flagged = 0;
+        _deleted = true;
+        return;
+    }
+    if (S_ISDIR(st->st_mode) != 0)
+        throw wibble::exception::Consistency(_path + " is a directory");
 
-	_mbox_mtime = st.st_mtime;
-	_mbox_size = st.st_size;
+    if (_deleted)
+        _deleted = false;
 
-//	if (!force && (s.st_size == mbox->file_size) && (s.st_mtime == mbox->file_mtime))
-//		return 0;
+    _mbox_mtime = st->st_mtime;
+    _mbox_size = st->st_size;
 
-	if (st.st_size == 0)
-	{
-		//mbox->file_size = 0;
-		//mbox->file_mtime = 0;
-		goto end2;
-	}
+//  if (!force && (s.st_size == mbox->file_size) && (s.st_mtime == mbox->file_mtime))
+//      return 0;
+
+    if (st->st_size == 0)
+    {
+        //mbox->file_size = 0;
+        //mbox->file_mtime = 0;
+        goto end2;
+    }
 
 	// TODO: lock the file
 	// Count the messages
@@ -203,13 +204,13 @@ end1:
 	gzclose(in);
 
 
-	// Restore the access time of the mailbox for other checking programs
-	struct timeval t[2];
-	t[0].tv_sec = st.st_atime;
-	t[0].tv_usec = 0;
-	t[1].tv_sec = st.st_mtime;
-	t[1].tv_usec = 0;
-	utimes(_path.c_str(), t);
+    // Restore the access time of the mailbox for other checking programs
+    struct timeval t[2];
+    t[0].tv_sec = st->st_atime;
+    t[0].tv_usec = 0;
+    t[1].tv_sec = st->st_mtime;
+    t[1].tv_usec = 0;
+    utimes(_path.c_str(), t);
 
 	// Return the values
 end2:
@@ -222,15 +223,17 @@ end2:
 
 static bool isMailbox(const std::string& pathname)
 {
-	// Perform consistency checks on the file
-	struct stat st;
-	stat(pathname, &st);
-	if (S_ISDIR(st.st_mode) != 0)
-		return false;
+    // Perform consistency checks on the file
+    std::auto_ptr<struct stat> st = sys::fs::stat(pathname);
+    if (!st.get())
+        return false;
 
-	// Can be an empty file
-	if (st.st_size == 0)
-		return true;
+    if (S_ISDIR(st->st_mode) != 0)
+        return false;
+
+    // Can be an empty file
+    if (st->st_size == 0)
+        return true;
 
 	// Check if it starts with a From line
 	gzFile in = gzopen(pathname.c_str(), "rb");
@@ -284,10 +287,10 @@ void Mailbox::enumerateFolders(const std::string& parent, MailFolderConsumer& co
 	if (S_ISDIR(st.st_mode) == 0)
 		return;
 
-	// Enumerate the Mailboxs in it
-	Directory dir(parent);
-	while (struct dirent *d = dir.read())
-	{
+    // Enumerate the Mailboxs in it
+    sys::fs::Directory dir(parent);
+    for (sys::fs::Directory::const_iterator d = dir.begin(); d != dir.end(); ++d)
+    {
 		if (strcmp(d->d_name, ".") == 0)
 			continue;
 		if (strcmp(d->d_name, "..") == 0)
@@ -303,7 +306,7 @@ void Mailbox::enumerateFolders(const std::string& parent, MailFolderConsumer& co
 		MailFolder f(accessFolder(candidate));
 		if (f)
 			cons.consume(f);
-	}
+    }
 }
 
 }
