@@ -1,7 +1,7 @@
 /*
  * Maildir folder access
  *
- * Copyright (C) 2004--2008  Enrico Zini <enrico@debian.org>
+ * Copyright (C) 2004--2011  Enrico Zini <enrico@debian.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,6 +20,8 @@
 
 #include <buffy/mailfolder/maildir.h>
 #include <wibble/exception.h>
+#include <wibble/sys/fs.h>
+#include <wibble/string.h>
 #include <buffy/utils/system.h>
 
 #include <sys/types.h>  // stat, opendir, readdir, utimes
@@ -80,21 +82,21 @@ bool Maildir::changed()
 	string path_new = _path + "/new";
 	string path_cur = _path + "/cur";
 
-	struct stat st_new;
-	if (!statIfFound(path_new, &st_new))
-		if (! _deleted)
-		{
-			_deleted = true;
-			return true;
-		}
+    std::auto_ptr<struct stat> st_new = sys::fs::stat(path_new);
+    if (!st_new.get())
+        if (! _deleted)
+        {
+            _deleted = true;
+            return true;
+        }
 
-	struct stat st_cur;
-	if (!statIfFound(path_cur, &st_cur))
-		if (! _deleted)
-		{
-			_deleted = true;
-			return true;
-		}
+    std::auto_ptr<struct stat> st_cur = sys::fs::stat(path_cur);
+    if (!st_cur.get())
+        if (! _deleted)
+        {
+            _deleted = true;
+            return true;
+        }
 
 	if (_deleted)
 	{
@@ -102,7 +104,7 @@ bool Maildir::changed()
 		return true;
 	}
 
-	return st_new.st_mtime > _new_mtime || st_cur.st_mtime > _cur_mtime;
+    return st_new->st_mtime > _new_mtime || st_cur->st_mtime > _cur_mtime;
 }
 
 void Maildir::updateStatistics()
@@ -116,70 +118,70 @@ void Maildir::updateStatistics()
 	string path_new = _path + "/new";
 	string path_cur = _path + "/cur";
 
-	// Perform consistency checks on the 'new' directory
-	struct stat st_new;
-	if (!statIfFound(path_new, &st_new))
-	{
+    // Perform consistency checks on the 'new' directory
+    std::auto_ptr<struct stat> st_new = sys::fs::stat(path_new);
+    if (!st_new.get())
+    {
 		_stat_total = 0;
 		_stat_unread = 0;
 		_stat_new = 0;
 		_stat_flagged = 0;
 		_deleted = true;
 		return;
-	}
-	if (S_ISDIR(st_new.st_mode) == 0)
-		throw wibble::exception::Consistency(path_new + " is not a directory");
+    }
+    if (S_ISDIR(st_new->st_mode) == 0)
+        throw wibble::exception::Consistency(path_new + " is not a directory");
 
-	// Perform consistency checks on the 'cur' directory
-	struct stat st_cur;
-	if (!statIfFound(path_cur, &st_cur))
-	{
+    // Perform consistency checks on the 'cur' directory
+    std::auto_ptr<struct stat> st_cur = sys::fs::stat(path_cur);
+    if (!st_cur.get())
+    {
 		_stat_total = 0;
 		_stat_unread = 0;
 		_stat_new = 0;
 		_stat_flagged = 0;
 		_deleted = true;
 		return;
-	}
-	if (S_ISDIR(st_cur.st_mode) == 0)
-		throw wibble::exception::Consistency(path_cur + " is not a directory");
+    }
+    if (S_ISDIR(st_cur->st_mode) == 0)
+        throw wibble::exception::Consistency(path_cur + " is not a directory");
 
 	if (_deleted)
 		_deleted = false;
 
-	_new_mtime = st_new.st_mtime;
-	_cur_mtime = st_cur.st_mtime;
+    _new_mtime = st_new->st_mtime;
+    _cur_mtime = st_cur->st_mtime;
 
-	/// Count messages in the 'new' directory
+    /// Count messages in the 'new' directory
 
-	// Count the files in the 'new' directory
-	{
-		Directory dir(path_new);
-		while (struct dirent* d = dir.read())
-		{
-			if (d->d_name[0] == '.') 
-				continue;
-			res_total++;
-			res_new++;
-		}
-	}
+    // Count the files in the 'new' directory
+    {
+        sys::fs::Directory dir(path_new);
+        for (sys::fs::Directory::const_iterator d = dir.begin(); d != dir.end(); ++d)
+        {
+            if (d->d_name[0] == '.') 
+                continue;
+            res_total++;
+            res_new++;
+        }
+    }
 
-	// Restore the access time of the mailbox for other checking programs
-	struct timeval t[2];
-	t[0].tv_sec = st_new.st_atime;
-	t[0].tv_usec = 0;
-	t[1].tv_sec = st_new.st_mtime;
-	t[1].tv_usec = 0;
-	utimes(path_new.c_str(), t);
+    // Restore the access time of the mailbox for other checking programs
+    struct timeval t[2];
+    t[0].tv_sec = st_new->st_atime;
+    t[0].tv_usec = 0;
+    t[1].tv_sec = st_new->st_mtime;
+    t[1].tv_usec = 0;
+    utimes(path_new.c_str(), t);
 
 
-	/// Count messages in the 'cur' directory
+    /// Count messages in the 'cur' directory
 
-	// Count the files in the 'cur' directory
-	{
-		Directory dir(path_cur);
-		while (struct dirent *d = dir.read())
-		{
+    // Count the files in the 'cur' directory
+    {
+        sys::fs::Directory dir(path_cur);
+        for (sys::fs::Directory::const_iterator d = dir.begin(); d != dir.end(); ++d)
+        {
 			if (d->d_name[0] == '.') 
 				continue;
 
@@ -200,15 +202,15 @@ void Maildir::updateStatistics()
 				res_unread++;
 			if (strchr(info, 'F') != 0)
 				res_flagged++;
-		}
-	}
+        }
+    }
 
-	// Restore the access time of the mailbox for other checking programs
-	t[0].tv_sec = st_cur.st_atime;
-	t[0].tv_usec = 0;
-	t[1].tv_sec = st_cur.st_mtime;
-	t[1].tv_usec = 0;
-	utimes(path_cur.c_str(), t);
+    // Restore the access time of the mailbox for other checking programs
+    t[0].tv_sec = st_cur->st_atime;
+    t[0].tv_usec = 0;
+    t[1].tv_sec = st_cur->st_mtime;
+    t[1].tv_usec = 0;
+    utimes(path_cur.c_str(), t);
 
 
 	// Return the values
@@ -221,32 +223,26 @@ void Maildir::updateStatistics()
 
 static bool isMaildir(const std::string& pathname)
 {
-	try {
-		// It must be a directory
-		struct stat st;
-		stat(pathname, &st);
-		if (S_ISDIR(st.st_mode) == 0)
-			return false;
+    try {
+        // It must be a directory
+        std::auto_ptr<struct stat> st = sys::fs::stat(pathname);
+        if (!st.get())
+            return false;
+        if (S_ISDIR(st->st_mode) == 0)
+            return false;
 
-		// It must contain cur, new and tmp subdirectories
-		const char* subdirs[3] = { "cur", "new", "tmp" };
-		for (int i = 0; i < 3; i++)
-		{
-			string subdir = concat(pathname, '/', subdirs[i]);
+        // It must contain cur, new and tmp subdirectories
+        const char* subdirs[3] = { "cur", "new", "tmp" };
+        for (int i = 0; i < 3; i++)
+            if (!sys::fs::isdir(str::joinpath(pathname, subdirs[i])))
+                return false;
 
-			struct stat st;
-			if (stat(subdir.c_str(), &st) != 0)
-				return false;
-			if (S_ISDIR(st.st_mode) == 0)
-				return false;
-		}
-
-		// It appears to be a maildir directory
-		return true;
-	} catch (wibble::exception::File& e) {
-		// If we can't even stat() it, then it's not a maildir
-		return false;
-	}
+        // It appears to be a maildir directory
+        return true;
+    } catch (wibble::exception::System& e) {
+        // If we can't even stat() it, then it's not a maildir
+        return false;
+    }
 }
 
 MailFolder Maildir::accessFolder(const std::string& path)
@@ -266,22 +262,24 @@ static void enumerateSubfolders(
 		MailFolderConsumer& cons,
 		InodeSet seen = InodeSet())
 {
-	try {
-		struct stat st;
-		try {
-			stat(parent, &st);
-		} catch (wibble::exception::File& e) {
-			// If we can't even stat() it, then we don't try to enumerate its subfolders
-			// FIXME: cerr << e.type() << ": " << e.fullInfo() << endl;
-			return;
-		}
-		// It must be a directory
-		if (S_ISDIR(st.st_mode) == 0)
-			return;
+    try {
+        std::auto_ptr<struct stat> st;
+        try {
+            st = sys::fs::stat(parent);
+        } catch (wibble::exception::File& e) {
+            // If we can't even stat() it, then we don't try to enumerate its subfolders
+            // FIXME: cerr << e.type() << ": " << e.fullInfo() << endl;
+            return;
+        }
+        if (!st.get()) return;
 
-		// Check that we aren't looping
-		if (seen.has(st.st_ino))
-			return;
+        // It must be a directory
+        if (S_ISDIR(st->st_mode) == 0)
+            return;
+
+        // Check that we aren't looping
+        if (seen.has(st->st_ino))
+            return;
 
 		if (isMaildir(parent))
 		{
@@ -289,10 +287,10 @@ static void enumerateSubfolders(
 			cons.consume(f);
 		}
 
-		// Recursively enumerate the Maildirs in the directory
-		Directory dir(parent);
-		while (struct dirent *d = dir.read())
-		{
+        // Recursively enumerate the Maildirs in the directory
+        sys::fs::Directory dir(parent);
+        for (sys::fs::Directory::const_iterator d = dir.begin(); d != dir.end(); ++d)
+        {
 			if (strcmp(d->d_name, ".") == 0)
 				continue;
 			if (strcmp(d->d_name, "..") == 0)
@@ -307,11 +305,11 @@ static void enumerateSubfolders(
 			enumerateSubfolders(
 					concat(parent, '/', d->d_name),
 					concat(name, '.', d->d_name),
-					cons, seen + st.st_ino);
-		}
-	} catch (wibble::exception::Generic& e) {
-		// FIXME: cerr << e.type() << ": " << e.fullInfo() << endl;
-	}
+					cons, seen + st->st_ino);
+        }
+    } catch (wibble::exception::Generic& e) {
+        // FIXME: cerr << e.type() << ": " << e.fullInfo() << endl;
+    }
 }
 
 void Maildir::enumerateFolders(const std::string& parent, MailFolderConsumer& cons)
