@@ -108,18 +108,23 @@ int main(int argc, char* argv[])
 			cout << "  important: " << cfg->view().important() << endl;
 
 			cout << "Locations:" << endl;
-			vector<string> locations = cfg->locations().get();
+			vector<string> locations = cfg->locations();
 			for (vector<string>::const_iterator i = locations.begin();
 					i != locations.end(); ++i)
 				cout << "  " << *i << endl;
 
 			cout << "Mail programs:" << endl;
-			std::vector<config::MailProgramNode> programs = cfg->mailPrograms().get();
-			for (vector<config::MailProgramNode>::const_iterator i = programs.begin();
+			std::vector<string> programs = cfg->mailPrograms();
+			for (vector<string>::const_iterator i = programs.begin();
 					i != programs.end(); ++i)
-				cout << "  " << i->name() << ": " << i->command() << (i->selected() ? " (selected)" : "") << endl;
-			config::MailProgramNode sel = cfg->mailPrograms().selected();
-			cout << "  sel: " << sel.name() << ": " << sel.command() << (sel.selected() ? " (selected)" : "") << endl;
+			{
+				config::MailProgram mp = cfg->mailProgram(*i);
+				cout << "  " << *i << ": text:" << mp.command("text")
+			             << " gui:" << mp.command("gui") << (mp.selected() ? " (selected)" : "") << endl;
+			}
+			config::MailProgram sel = cfg->selectedMailProgram();
+			cout << "  sel: " << sel.name() << ": text:" << sel.command("text")
+		             << " gui:" << sel.command("gui") << (sel.selected() ? " (selected)" : "") << endl;
 
 			//cout << "Applications:" << endl;
 			//cout << "Folders:" << endl;
